Module: Strelka::App::Sessions::ClassMethods
- Defined in:
- lib/strelka/app/sessions.rb
Overview
Class methods and instance variables to add to classes with sessions.
Instance Method Summary collapse
-
#inherited(subclass) ⇒ Object
Extension callback -- add instance variables to extending objects.
-
#session_namespace(new_namespace = nil) ⇒ Object
Get/set the key that will determine which namespace in the session object the application will see.
Instance Method Details
#inherited(subclass) ⇒ Object
Extension callback -- add instance variables to extending objects.
115 116 117 118 |
# File 'lib/strelka/app/sessions.rb', line 115 def inherited( subclass ) super subclass.instance_variable_set( :@session_namespace, @session_namespace ) end |
#session_namespace(new_namespace = nil) ⇒ Object
Get/set the key that will determine which namespace in the session object the application will see.
123 124 125 126 127 |
# File 'lib/strelka/app/sessions.rb', line 123 def session_namespace( new_namespace=nil ) @session_namespace = new_namespace if new_namespace @session_namespace ||= self.default_appid return @session_namespace end |